/*
Copyright 2002-2005 The Members of the OSS through Java(TM) Initiative.
All rights reserved. Use is subject to license terms.
 */
package ossj.common.cbe.schedule;
import ossj.common.Utils;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import javax.oss.cbe.schedule.AnnualSchedule;


/**
 * An implementation class for the <CODE>javax.oss.cbe.schedule.AnnualSchedule</CODE> interface.
 * 
 * 
 * @author OSS through Java Initiative, Vincent Perrot Sun Microsystems Inc.
 * @version 1.4
 * @see javax.oss.cbe.schedule.AnnualSchedule
 * @see javax.oss.cbe.schedule.Schedule
 * @since August 2005
 */
public class AnnualScheduleImpl implements AnnualSchedule {
    private boolean[] isMonthActive = new boolean[12];
    private TimeZone timeZone = null;
    
    /**
     * Creates a new AnnualScheduleImpl object.
     */
    public AnnualScheduleImpl() {
        // All Months disabled by default.
        for (int i = 0; i < 12; i++) {
            isMonthActive[i] = false;
        }
    }
    
    /**
     * Checks if the schedule is active or not.
     *
     * <p>
     * If the current Month of Year is within the defined schedule this method will return
     * true. If the current Month of Year is outside the schedule false will be returned.
     *
     *@return boolean Returns true if the schedule is active, else false.
     */
    public boolean isActive() {
        boolean monthActive = false;
        GregorianCalendar currentDateAndTime;
        
        if (timeZone != null) {
            currentDateAndTime = new GregorianCalendar(timeZone);
        } else {
            currentDateAndTime = new GregorianCalendar();
        }
        
        if (isMonthActive[currentDateAndTime.get(Calendar.MONTH)] == true) {
            monthActive = true;
        }
        
        return monthActive;
    }
    
    /**
     * Gets the time zone of the Monthly schedule.
     *
     *@return TimeZone The time zone of the Monthly schedule.
     */
    public java.util.TimeZone getTimeZone() {
        return (TimeZone) timeZone;
    }
    
    /**
     * Sets the time zone for the Monthly schedule.
     *
     * <p>
     * If the time zone is not set the local time zone will be used.
     *
     *@param tz Time zone of the Monthly schedule.
     */
    public void setTimeZone(java.util.TimeZone tz) {
        timeZone = tz;
    }
    
    /**
     * Sets all Months of the Year to be active in the Month of Annual schedule.
     */
    public void setAllMonthsActive() {
        for (int i = 1; i < 7; i++) {
            isMonthActive[i] = true;
        }
    }
    
    /**
     * Return true if the specified Month is active.
     * @see java.util.Calendar
     * @param month is the integer representing the Month of the Year as specified in the java.util.Calendar
     */
    public boolean isActiveMonth(int month){
        if (month < 0 || month > 11)
            throw new java.lang.IllegalArgumentException("Month shall be in the range [0-11], current ="+month);
        else
            return isMonthActive[month];
    }
    
    /**
     * Return true if the specified Month is active.
     * @see java.util.Calendar
     * @param month is the integer representing the Month of the Year as specified in the java.util.Calendar
     * @param active true to activate the given Month
     */
    public void setActiveMonth(int month, boolean active){
        if (month < 0 || month > 11)
            throw new java.lang.IllegalArgumentException("Month shall be in the range [0-11], current ="+month);
        else
            isMonthActive[month]=active;
    }
    
    /**
     * Return the array of activity for the Year.
     * @see java.util.Calendar
     * The index of the array corresponds to the Month of the Year as specified in the java.util.Calendar
     * @return the activity array
     */
    public boolean[] getActiveMonths(){
        return isMonthActive;
    }
    
    /**
     * Sets the activity for the Year.
     * @see java.util.Calendar
     * The index of the array corresponds to the Month of the Year as specified in the java.util.Calendar
     * @param activities the activity array
     */
    public void setActiveMonths(boolean[] activities){
        isMonthActive = activities;
    }
    
    /**
     * Creates a new object of the same class and with the same contents as this object.
     * @return a clone of this instance.
     */
    public Object clone() {
        AnnualScheduleImpl clone = null;
        
        try {
            clone = (AnnualScheduleImpl) super.clone();
            
            if (timeZone != null) {
                clone.setTimeZone(getTimeZone());
            }
        } catch( CloneNotSupportedException ex) {
            throw new RuntimeException("MonthlyScheduleImpl: Unable to clone this object: "
                    + ex.getMessage());
        }
        
        return clone;
    }
    
    /**
     * Checks whether two AnnualSchedule are equal.
     * The result is true if and only if the argument is not null
     * and is an AnnualSchedule object that has the arguments.
     *
     * @param value the Object to compare with this AnnualSchedule
     * @return if the objects are equal; false otherwise.
     */
    public boolean equals(Object value) {
        if (this == value) {
            return true;
        }
        
        if ((value != null) && (value instanceof AnnualSchedule)) {
            AnnualSchedule argVal = (AnnualSchedule) value;
            
            if (this.getActiveMonths().length != argVal.getActiveMonths().length) {
                return false;
            }
            for (int i = this.getActiveMonths().length - 1; i > 0 ; i--){
                if (this.isActiveMonth(i) != argVal.isActiveMonth(i)) return false;
            }
            
            if (!Utils.compareAttributes(getTimeZone(), argVal.getTimeZone())) {
                return false;
            }
            
            return true;
        } else {
            return false;
        }
    }
}
