/*
Copyright 2002-2005 The Members of the OSS through Java(TM) Initiative.
All rights reserved. Use is subject to license terms.
 */
package ossj.common.cbe.schedule;
import ossj.common.Utils;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;

import javax.oss.cbe.schedule.MonthlySchedule;


/**
 * An implementation class for the <CODE>javax.oss.cbe.schedule.MonthlySchedule</CODE> interface.
 *
 * @see javax.oss.cbe.schedule.MonthlySchedule
 * @see javax.oss.cbe.schedule.Schedule
 *
 * @author OSS through Java Initiative, Vincent Perrot Sun Microsystems Inc.
 * @version 1.4
 * @since August 2005
 */
public class MonthlyScheduleImpl implements MonthlySchedule {
    private boolean[] isWeekActive = new boolean[7];
    private TimeZone timeZone = null;
    
    /**
     * Creates a new MonthlyScheduleImpl object.
     */
    public MonthlyScheduleImpl() {
        // All Weeks disabled by default.
        for (int i = 0; i < 7; i++) {
            isWeekActive[i] = false;
        }
    }
    
    /**
     * Checks if the schedule is active or not.
     *
     * <p>
     * If the current Week of Month is within the defined schedule this method will return
     * true. If the current Week of Month is outside the schedule false will be returned.
     *
     *@return boolean Returns true if the schedule is active, else false.
     */
    public boolean isActive() {
        boolean weekActive = false;
        GregorianCalendar currentDateAndTime;
        
        if (timeZone != null) {
            currentDateAndTime = new GregorianCalendar(timeZone);
        } else {
            currentDateAndTime = new GregorianCalendar();
        }
        
        if (isWeekActive[currentDateAndTime.get(Calendar.WEEK_OF_MONTH)] == true) {
            weekActive = true;
        }
        
        return weekActive;
    }
    
    /**
     * Gets the time zone of the monthly schedule.
     *
     *@return TimeZone The time zone of the monthly schedule.
     */
    public java.util.TimeZone getTimeZone() {
        return (TimeZone) timeZone;
    }
    
    /**
     * Sets the time zone for the monthly schedule.
     *
     * <p>
     * If the time zone is not set the local time zone will be used.
     *
     *@param tz Time zone of the monthly schedule.
     */
    public void setTimeZone(java.util.TimeZone tz) {
        timeZone = tz;
    }
    
    /**
     * Sets all Weeks of the month to be active in the Week of monthly schedule.
     */
    public void setAllWeeksActive() {
        for (int i = 1; i < 7; i++) {
            isWeekActive[i] = true;
        }
    }
    
    /**
     * Return true if the specified Week is active.
     * @see java.util.Calendar
     * @param week is the integer representing the Week of the month as specified in the java.util.Calendar
     */
    public boolean isActiveWeek(int week){
        if (week < 1 || week > 6)
            throw new java.lang.IllegalArgumentException("Week of the month shall be in the range [1-5], current ="+week);
        else
            return isWeekActive[week];
    }
    
    /**
     * Return true if the specified Week is active.
     * @see java.util.Calendar
     * @param week is the integer representing the Week of the month as specified in the java.util.Calendar
     * @param active true to activate the given week
     */
    public void setActiveWeek(int week, boolean active){
        if (week < 1 || week > 6)
            throw new java.lang.IllegalArgumentException("Week of the month shall be in the range [1-5], current ="+week);
        else
            isWeekActive[week]=active;
    }
    
    /**
     * Return the array of activity for the week.
     * @see java.util.Calendar
     * The index of the array corresponds to the Week of the month as specified in the java.util.Calendar
     * @return the activity array
     */
    public boolean[] getActiveWeeks(){
        return isWeekActive;
    }
    
    /**
     * Sets the activity for the week.
     * @see java.util.Calendar
     * The index of the array corresponds to the Week of the month as specified in the java.util.Calendar
     * @param activities the activity array
     */
    public void setActiveWeeks(boolean[] activities){
        isWeekActive = activities;
    }
    
    /**
     * Creates a new object of the same class and with the same contents as this object.
     * @return a clone of this instance.
     */
    public Object clone() {
        MonthlyScheduleImpl clone = null;
        
        try {
            clone = (MonthlyScheduleImpl) super.clone();
            
            if (timeZone != null) {
                clone.setTimeZone(getTimeZone());
            }
        } catch( CloneNotSupportedException ex) {
            throw new RuntimeException("MonthlyScheduleImpl: Unable to clone this object: "
                    + ex.getMessage());
        }
        
        return clone;
    }
    
    /**
     * Checks whether two MonthlySchedule are equal.
     * The result is true if and only if the argument is not null
     * and is an MonthlySchedule object that has the arguments.
     *
     * @param value the Object to compare with this MonthlySchedule
     * @return if the objects are equal; false otherwise.
     */
    public boolean equals(Object value) {
        if (this == value) {
            return true;
        }
        
        if ((value != null) && (value instanceof MonthlySchedule)) {
            MonthlySchedule argVal = (MonthlySchedule) value;
            
            if (this.getActiveWeeks().length != argVal.getActiveWeeks().length) {
                return false;
            }
            for (int i = this.getActiveWeeks().length - 1; i > 0 ; i--){
                if (this.isActiveWeek(i) != argVal.isActiveWeek(i)) return false;
            }
            
            if (!Utils.compareAttributes(getTimeZone(), argVal.getTimeZone())) {
                return false;
            }
            
            return true;
        } else {
            return false;
        }
    }
}
